@@ -318,16 +318,11 @@ create_launcher (const char *parameters, GnomeDesktopItem *ditem)
 	if (!default_app_pixmap)
 		default_app_pixmap = panel_pixmap_discovery ("gnome-unknown.png",
 							     FALSE /* fallback */);
-#ifdef LAUNCHER_DEBUG
-	printf ("Creating Launcher %s\n", parameters);
-#endif
 	if (ditem == NULL) {
 		if (parameters == NULL) {
 			return NULL;
 		}
-#ifdef LAUNCHER_DEBUG
-	printf ("Creating gnome_desktop_item_new_from_uri\n");
-#endif
+
 		ditem = gnome_desktop_item_new_from_uri (parameters,
 							 0 /* flags */,
 							 NULL /* error */);
@@ -494,6 +489,8 @@ properties_apply (Launcher *launcher)
 
 	/* Setup the button look */
 	setup_button (launcher);
+
+	launcher_save_to_gconf (launcher, launcher->info->gconf_key);
 }
 
 static void
@@ -511,21 +508,6 @@ properties_close_callback(GtkWidget *widget, gpointer data)
 	panel_config_sync_schedule ();
 }
 
-static void
-window_response (GtkWidget *w, int response, gpointer data)
-{
-	Launcher *launcher = data;
-
-	if (response == GTK_RESPONSE_HELP) {
-		panel_show_help ("launchers", NULL);
-	} else if (response == REVERT_BUTTON) { /* revert */
-		gnome_ditem_edit_set_ditem (GNOME_DITEM_EDIT (launcher->dedit),
-					    launcher->revert_ditem);
-	} else {
-		gtk_widget_destroy (w);
-	}
-}
-
 static void
 launcher_changed (GtkObject *dedit, gpointer data)
 {
@@ -534,22 +516,76 @@ launcher_changed (GtkObject *dedit, gpointer data)
 	properties_apply (launcher);
 }
 
+static void
+set_revert_insensitive (GtkWidget *button,	
+			gpointer   dummy)
+{
+     gtk_widget_set_sensitive (button, FALSE);
+}
+
+static void
+set_revert_sensitive (GtkWidget *button,
+		      gpointer   dummy)
+{
+     gtk_widget_set_sensitive (button, TRUE);
+}
+
+static void
+window_response (GtkWidget *w, int response, gpointer data)
+{
+	Launcher *launcher = data;
+
+	if (response == GTK_RESPONSE_HELP) {
+		panel_show_help ("launchers", NULL);
+	} else if (response == REVERT_BUTTON) { /* revert */
+		if (launcher->ditem != NULL)
+			gnome_desktop_item_unref (launcher->ditem);
+		launcher->ditem = gnome_desktop_item_copy (launcher->revert_ditem);
+
+		/* We want to ignore the "changed" signal first */ 
+		g_signal_handlers_disconnect_by_func (
+				   G_OBJECT (GNOME_DITEM_EDIT(launcher->dedit)),
+				   G_CALLBACK(launcher_changed),
+				   launcher);
+
+		gnome_ditem_edit_set_ditem (GNOME_DITEM_EDIT (launcher->dedit),
+					    launcher->revert_ditem);
+
+		/* connect the launcher_changed () callback for the "changed"
+		 * signal again
+		 */
+		g_signal_connect (G_OBJECT (launcher->dedit), "changed",
+				    G_CALLBACK (launcher_changed),
+				    launcher);
+
+		properties_apply (launcher);
+
+	} else {
+		gtk_widget_destroy (w);
+	}
+}
+
 static GtkWidget *
 create_properties_dialog (Launcher *launcher)
 {
 	GtkWidget *dialog;
+        GtkWidget *help;
+        GtkWidget *close;
+        GtkWidget *revert;
 
-	/* watch the enum at the top of the file */
-	dialog = gtk_dialog_new_with_buttons (_("Launcher properties"),
-					      NULL /* parent */,
-					      0 /* flags */,
-					      GTK_STOCK_HELP,
-					      GTK_RESPONSE_HELP,
-					      GTK_STOCK_REVERT_TO_SAVED,
-					      REVERT_BUTTON,
-					      GTK_STOCK_CLOSE,
-					      GTK_RESPONSE_CLOSE,
-					      NULL);
+	dialog = gtk_dialog_new ();
+
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Launcher properties"));
+
+	help = gtk_dialog_add_button (
+			GTK_DIALOG (dialog), GTK_STOCK_HELP, GTK_RESPONSE_HELP);
+
+	revert = gtk_dialog_add_button (
+			GTK_DIALOG (dialog), GTK_STOCK_REVERT_TO_SAVED, REVERT_BUTTON);
+        gtk_widget_set_sensitive (revert, FALSE);
+
+	close = gtk_dialog_add_button (
+			GTK_DIALOG (dialog), GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
 
 	launcher->dedit = gnome_ditem_edit_new ();
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox),
@@ -565,19 +601,27 @@ create_properties_dialog (Launcher *launcher)
 	gnome_ditem_edit_set_ditem (GNOME_DITEM_EDIT (launcher->dedit),
 				    launcher->ditem);
 
-	g_signal_connect (G_OBJECT (launcher->dedit), "changed",
-			    G_CALLBACK (launcher_changed),
-			    launcher);
+	g_signal_connect (launcher->dedit, "changed",
+			  G_CALLBACK (launcher_changed),
+			  launcher);
 
-	g_signal_connect (G_OBJECT (dialog), "destroy",
-			    G_CALLBACK (properties_close_callback),
-			    launcher);
+        g_signal_connect_swapped (launcher->dedit, "changed",
+				  G_CALLBACK (set_revert_sensitive),
+				  revert);
+
+	g_signal_connect (dialog, "destroy",
+			  G_CALLBACK (properties_close_callback),
+			  launcher);
 
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_CLOSE);
 
-	g_signal_connect (G_OBJECT (dialog), "response",
-			    G_CALLBACK (window_response),
-			    launcher);
+	g_signal_connect (dialog, "response",
+			  G_CALLBACK (window_response),
+			  launcher);
+
+	g_signal_connect (revert, "clicked",
+			  G_CALLBACK (set_revert_insensitive),
+			  NULL);
 
 	gnome_ditem_edit_grab_focus (GNOME_DITEM_EDIT (launcher->dedit));
 
@@ -606,40 +650,40 @@ launcher_save_to_gconf (Launcher   *launcher,
 	if (location) {
 		GConfClient *client;
 		const char  *profile;
-		char        *temp_key;
+		const char  *temp_key;
 
 		client  = panel_gconf_get_client ();
-		profile = session_get_current_profile ();
+		profile = panel_gconf_get_profile ();
 
-		temp_key = panel_gconf_objects_profile_get_full_key (profile, gconf_key, "launcher-location");
+		temp_key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile,
+						 gconf_key, "launcher-location");
 		gconf_client_set_string (client, temp_key, location, NULL);
-		g_free (temp_key);
 	}
 }
 
 void
 launcher_load_from_gconf (PanelWidget *panel_widget,
 			  gint         position,
-			  const char  *gconf_key,
-			  gboolean     use_default)
+			  const char  *gconf_key)
 {
-        GConfClient *client;
-        char        *temp_key;
-        char        *launcher_location;
+	GConfClient *client;
+	const char  *profile;
+	const char  *temp_key;
+	char        *launcher_location;
 
-        g_return_if_fail (panel_widget);
-        g_return_if_fail (gconf_key);
+	g_return_if_fail (panel_widget);
+	g_return_if_fail (gconf_key);
 
-        client  = panel_gconf_get_client ();
-	/* FIXME : We need to do checking of screen dimensions for default */
-        temp_key = use_default ? panel_gconf_objects_default_profile_get_full_key ("medium", gconf_key, "launcher-location") :
-		panel_gconf_objects_profile_get_full_key (session_get_current_profile (), gconf_key, "launcher-location");
-        launcher_location = gconf_client_get_string (client, temp_key, NULL);
-        g_free (temp_key);
+	client  = panel_gconf_get_client ();
+	profile = panel_gconf_get_profile ();
+
+	temp_key = panel_gconf_full_key (
+			PANEL_GCONF_OBJECTS, profile, gconf_key, "launcher-location");
+	launcher_location = gconf_client_get_string (client, temp_key, NULL);
 
 	load_launcher_applet (launcher_location, panel_widget, position, TRUE, gconf_key);
 
-        g_free (launcher_location);
+	g_free (launcher_location);
 }
 
 Launcher *
@@ -650,7 +694,7 @@ load_launcher_applet_full (const char       *params,
 			   gboolean          exactpos,
 			   const char       *gconf_key)
 {
-	Launcher   *launcher;
+	Launcher *launcher;
 
 	launcher = create_launcher (params, ditem);
 
@@ -769,7 +813,7 @@ ditem_set_icon (GnomeDesktopItem *ditem, const char *icon)
 	if (icon != NULL &&
 	    icon[0] != G_DIR_SEPARATOR) {
 		char *full = gnome_desktop_item_find_icon (icon,
-							   20 /* desired size */,
+							   48 /* desired size */,
 							   0 /* flags */);
 		if (full != NULL) {
 			gnome_desktop_item_set_string (ditem,
@@ -867,7 +911,7 @@ launcher_file_name (const char *location)
 
 	g_return_val_if_fail (location!= NULL, NULL);
 
-	tmp = gnome_util_home_file (PANEL_CONFIG_PATH "launchers");
+	tmp = gnome_util_home_file (PANEL_LAUNCHERS_PATH);
 	/* Make sure the launcher directory exists */
 	if (!g_file_test (tmp, G_FILE_TEST_EXISTS)) {
 		panel_ensure_dir (tmp);
